"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _schemaUtils = require("schema-utils");

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _someLoaderUtils = require("@zougt/some-loader-utils");

var _pluginOptions = _interopRequireDefault(require("./plugin-options.json"));

var _arbitraryModeApply = require("./apply/arbitraryModeApply");

var _presetModeApply = require("./apply/presetModeApply");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable no-param-reassign */
class ThemeCssExtractWebpackPlugin {
  constructor(options = {}) {
    (0, _schemaUtils.validate)(_pluginOptions.default, options, {
      name: 'Extract Theme CSS Plugin',
      baseDataPath: 'options'
    });
    this.userOptions = Object.assign({
      multipleScopeVars: [],
      outputDir: '/',
      defaultScopeName: '',
      extract: true,
      removeCssScopeName: false,
      // type: function
      customThemeCssFileName: null,
      themeLinkTagId: 'theme-link-tag',
      themeLinkTagAppend: false,
      // 以下是任意主题模式的参数 arbitraryMode:true 有效
      arbitraryMode: false,
      // 默认主题色，必填
      defaultPrimaryColor: '',
      // style标签的id
      styleTagId: 'custom-theme-tagid',
      // boolean || "head" || "body"
      InjectDefaultStyleTagToHtml: true,
      includeStyleWithColors: [],
      hueDiffControls: {
        low: 0,
        high: 0
      },
      customThemeOutputPath: ''
    }, options);

    if (this.userOptions.customThemeOutputPath && !_fsExtra.default.existsSync(this.userOptions.customThemeOutputPath)) {
      _fsExtra.default.outputFileSync(this.userOptions.customThemeOutputPath, `function setCustomTheme() {
        return null;
      }\nexport default setCustomTheme;`);
    }

    (0, _someLoaderUtils.removeThemeFiles)();
    (0, _someLoaderUtils.createPulignParamsFile)({
      extract: this.userOptions.extract,
      includeStyleWithColors: this.userOptions.includeStyleWithColors,
      arbitraryMode: this.userOptions.arbitraryMode,
      customThemeOutputPath: this.userOptions.customThemeOutputPath
    });
  }

  apply(compiler) {
    if (this.userOptions.arbitraryMode) {
      _arbitraryModeApply.arbitraryModeApply.call(this, compiler);

      return;
    }

    _presetModeApply.presetModeApply.call(this, compiler);
  }

}

var _default = ThemeCssExtractWebpackPlugin;
exports.default = _default;